function [GNSS_meas_obtained, GNSS_pos, GNSS_var] = GNSS( vessel_pos, k, N, GNSS_info, gnss_number, fault_signal)
    
    global time 
    persistent GNSS_std_1 GNSS_std_2 GNSS_std_3 current_var GNSS_normal_std_weight ....
        GNSS_high_noise_std_weight GNSS_bias GNSS_bias_drift GNSS_prev_pos GNSS_1_prev_pos GNSS_2_prev_pos GNSS_3_prev_pos wild_points c_wild_points			
    
    if isempty(GNSS_std_1) || isempty(GNSS_std_2) || isempty( GNSS_std_3 )      
        GNSS_std_1 = normrnd(0, 1, N + 1, 2);
        GNSS_std_1(:,3) = normrnd(0, 1.1, N + 1, 1);
        GNSS_std_2 = normrnd(0, 1, N + 1, 2);
        GNSS_std_2(:,3) = normrnd(0, 1.1, N + 1, 1);
        GNSS_std_3 = normrnd(0, 1, N + 1, 2);
        GNSS_std_3(:,3) = normrnd(0, 1.1, N + 1, 1);
    end
    if isempty(GNSS_1_prev_pos) || isempty(GNSS_2_prev_pos) || isempty(GNSS_3_prev_pos)
       GNSS_1_prev_pos = [0 0 0]'; GNSS_2_prev_pos = [0 0 0]'; GNSS_3_prev_pos = [0 0 0]'; 
    end
    if isempty(GNSS_normal_std_weight) || isempty(GNSS_normal_std_weight) || isempty(GNSS_bias)
        GNSS_normal_std_weight = GNSS_info.GNSS_normal_std_weight;
        GNSS_high_noise_std_weight = GNSS_info.GNSS_high_noise_std_weight;
        GNSS_bias = GNSS_info.GNSS_bias;    
    end
    if isempty(wild_points) 
        wild_points (1,:) = 1.9098*[11 -10 -12.5];
        wild_points (2,:) = 1.5093*[11 10 -12.5];
        wild_points (3,:) = 2.1754*[-11 -10 -12.5];
        wild_points (4,:) = 1.4158*[11 10 12.5];
        wild_points (5,:) = 1.2154*[-11 10 -12.5];
        wild_points (6,:) = 1.4863*[-11 -10 12.5];
        wild_points (7,:) = 1.7701*[11 -10 -12.5];
        wild_points (8,:) = 1.2245*[11 10 -12.5];  
        wild_points (9,:) = 1.3040*[-11 -10 12.5];
        wild_points (10,:) = 1.8072*[-11 10 -12.5];
        wild_points (11,:) = 2.8218*[-11 10 12.5];
        wild_points (12,:) = 2.6404*[11 10 -12.5];
        wild_points (13,:) = 1.5037*[11 -10 -12.5];
        c_wild_points = 1;
    end

    if isempty(current_var), current_var = [0 0 0]'; end
    if isempty(GNSS_bias_drift), GNSS_bias_drift = [0 0 0]'; end
    if isempty(GNSS_prev_pos), GNSS_prev_pos = [0 0 0]'; end
    
    switch gnss_number
        case 1
            std = GNSS_std_1(k,:)';
            GNSS_prev_pos = GNSS_1_prev_pos;
        case 2
            std = GNSS_std_2(k,:)';
            GNSS_prev_pos = GNSS_2_prev_pos;
        case 3
            std = GNSS_std_3(k,:)';
            GNSS_prev_pos = GNSS_3_prev_pos;
    end
    % bias on the GNSS can be inserted
    % should be read from GNSS config file
    switch fault_signal
        case 0
            % normal operation
            GNSS_meas_obtained = 1;
            std = GNSS_normal_std_weight*std;
            GNSS_pos = vessel_pos + std;
        case 1
            % freeze, no update
            GNSS_meas_obtained = 1;
            GNSS_pos = GNSS_prev_pos;
        case 2 
            % large std
            GNSS_meas_obtained = 1;
            std = GNSS_high_noise_std_weight*std;
            GNSS_pos = vessel_pos + std;
        case 3 
            % wild point
            GNSS_meas_obtained = 1;
            bias = wild_points(c_wild_points,:)';
            GNSS_pos = vessel_pos + std + bias;
            c_wild_points = c_wild_points + 1;
            if c_wild_points > 13
                c_wild_points = 1;
            end            
        case 4
            % bias
            GNSS_meas_obtained = 1;
            GNSS_pos = vessel_pos + std + GNSS_bias;
        case 5
            % random lagre bias
            GNSS_meas_obtained = 1;
            bias = [10*randn 10*randn 10*randn]';
            GNSS_pos = vessel_pos + std + bias;
        case 6
            % drift
            GNSS_meas_obtained = 1;
            std = GNSS_normal_std_weight*std;
            GNSS_pos = vessel_pos + GNSS_bias_drift + std;
%             GNSS_bias_drift = GNSS_bias_drift + 0.005*[2 3 5]';
            GNSS_bias_drift = GNSS_bias_drift + 0.01*[2 3 5]';
        case 7
            % small GNSS drift
            GNSS_meas_obtained = 1;
            std = GNSS_normal_std_weight*std;
            GNSS_pos = vessel_pos + GNSS_bias_drift + std;
            GNSS_bias_drift = GNSS_bias_drift + 0.001*[4 2 3]';
        case 8
            % drift z axis
            GNSS_meas_obtained = 1;
            std = GNSS_normal_std_weight*std;
            GNSS_pos = vessel_pos + [0 0 10]' + std;
        case 9
            % scintillations or other failure that GNSS is not available
            GNSS_meas_obtained = 0;
            GNSS_pos = [ 0 0 0 ]';
        case 1000 
            % test case. ideal meaurment
            GNSS_meas_obtained = 1;
            GNSS_pos = vessel_pos;
    end
    switch gnss_number
        case 1
            GNSS_1_prev_pos = GNSS_pos;
        case 2
            GNSS_2_prev_pos = GNSS_pos;
        case 3
            GNSS_2_prev_pos = GNSS_pos;
    end
    GNSS_var = std.^2;
end