function [voting_status_out, k_1, k_2, k_3] = GNSS_voting( GNSS_1, GNSS_2, GNSS_3, threshold, k_1_prev, k_2_prev, k_3_prev)
    % GNSS_voting
    % voting status = 1. No descipancy
    % voting status = 2. Reduced redundancy. Two sensor operational. One sensor reported erroneous
    % voting status = 3. One sensor operational. Two sensor reported erroneous
    % voting status = 4. Two sensor operational. One sensor discarded in voting.
    % voting status = 5. Two sensor operational. Discrepancy between sensors detected
    persistent voting_status 
    if isempty( voting_status), voting_status = struct('status',1 ,'msg', '123'); end
    
    k_1 = k_1_prev;
    k_2 = k_2_prev;
    k_3 = k_3_prev;
    GNSS_1 = GNSS_1*k_1_prev;
    GNSS_2 = GNSS_2*k_2_prev;
    GNSS_3 = GNSS_3*k_3_prev;
    
    vote_type = k_1_prev + k_2_prev + k_3_prev;
    
    if k_1_prev == 1 && k_2_prev == 1
        [x1_x2, y_1_y_2] = circcirc(GNSS_1(1), GNSS_1(2), threshold, GNSS_2(1), GNSS_2(2), threshold);
    end
    if k_1_prev == 1 && k_3_prev == 1
        [x1_x3, y_1_y_3] = circcirc(GNSS_1(1), GNSS_1(2), threshold, GNSS_3(1), GNSS_3(2), threshold);
    end
    if k_2_prev == 1 && k_3_prev == 1
        [x2_x3, y_2_y_3] = circcirc(GNSS_2(1), GNSS_2(2), threshold, GNSS_3(1), GNSS_3(2), threshold);
    end
    
    switch vote_type
        % vote: Full. Possisible to exclude
        case 3

            if isnan( norm(x1_x2) ) && isnan( norm(x1_x3) )
               k_1 = 0;
               voting_status.status = 4;
               voting_status.msg = '1';
            elseif isnan( norm(x1_x2) ) && isnan( norm(x2_x3) )
               k_2 = 0;
               voting_status.status = 4;
               voting_status.msg = '2';
            elseif isnan( norm(x1_x3) ) && isnan( norm(x2_x3) )
               k_3 = 0;
               voting_status.status = 4;
               voting_status.msg = '3';
            else
               voting_status.status = 1;
               voting_status.msg = '123'; 
            end
        % vote: Reduced. Possisible only to resport desripancy 
        case 2
            if (k_1 + k_2 == 2) && isnan( norm(x1_x2) )
                voting_status.status = 5;
                voting_status.msg = '12';
            elseif k_1 + k_3 == 2 && isnan( norm(x1_x3) )
                voting_status.status = 5;
                voting_status.msg = '13';
            elseif k_2 + k_3 == 2 && isnan( norm(x2_x3) )
                voting_status.status = 5;
                voting_status.msg = '23';
            else
                voting_status.status = 2;
                if k_1 + k_2 == 2
                    msg = '12';
                elseif k_1 + k_3 == 2
                    msg = '13';
                elseif k_2 + k_3 == 2
                    msg = '23';
                end
                voting_status.msg = msg;
            end
        % vote: Not possible    
        case 1
            voting_status.status = 3;
            if k_1 == 1
                voting_status.msg = '1';
            elseif k_2 == 2
                voting_status.msg = '2';
            elseif k_3 == 1
                voting_status.msg = '3';
            end
    end
    voting_status_out = voting_status;
end