function C = M_RB_and_Omega_b2Coriolis(M_RB, omega_b_bn, r_g_2_b)
% M_RB_AND_OMEGA_B2_CORIOLIS
%
% C = M_RB_and_Omega_b2Coriolis(M_RB, omega_b_bn, r_b_g) computes the 
% 6x6 Coriolis-centripetal matrix C from the rigid-body mass and inertia  
% together with the angular velocity since the Coriolis is paramtrized 
% indipendtent of linear velocity
% 
% Author:    Torleiv Hland Bryne
% Date:      Spring 2013
% Revisions: 
% ________________________________________________________________
%

m = M_RB(1,1); % Rigid Body mass, m = M_RB(1,1) = M_RB(2,2) = M_RB(3,3)
I_b = M_RB(4:6,4:6); % Moment of Inertia, I_b

S_omega = Smtrx(omega_b_bn);
S_omega_I_b = Smtrx(I_b*omega_b_bn);
S_arm_g2b = Smtrx(r_g_2_b);

C = [   m*S_omega,              - m*S_omega*S_arm_g2b;
        m*S_arm_g2b*S_omega,    -S_omega_I_b ];
