function bias_plot( time, bias_hat_gyro, bias_hat_acc, biases, xi_hat, sigma,....
    bias_gnss, gnss_z_bias_status_out, gnss_bias_threshold, y_1_out, x_out)

global s_Folder;

set(0,'defaulttextinterpreter','Latex')

bias_gyro = rad2deg(biases.b_gyro_b);
bias_acc = biases.b_a_b;
bias_hat_gyro_deg = rad2deg(bias_hat_gyro);

s_xlabel = 'Time [s]';
% s_legend = {'$\hat{b}_x$', '$b_x$', '$\hat{b}_y$', '$b_y$', '$\hat{b}_z$', '$b_z$'};
s_legend = {'$x$', '$y$', '$z$'};

%% gryo bias
h = figure;
resize_fig( 'bias_plot' );
s_title = 'Gyroscope Bias Estimates';
s_y_label = '$\boldmath{\hat{b}}^b_g$[deg/s]';
s_fig_file_name = 'pva_gyro_bias_est';

h1 = plot(time, bias_hat_gyro_deg(:,1), 'b');
hold on
plot([time(1), time(end)], [bias_gyro(1), bias_gyro(1)], '--b') 
h2 = plot(time, bias_hat_gyro_deg(:,2),'r');
plot([time(1), time(end)], [bias_gyro(2), bias_gyro(2)], '--r')
h3 = plot(time, bias_hat_gyro_deg(:,3),'g');
plot([time(1), time(end)], [bias_gyro(3), bias_gyro(3)], '--g') 
hold off;
legend([h1, h2, h3],s_legend, 'Interpreter','Latex','location','best');
%title(s_title)
xlabel(s_xlabel, 'Interpreter','Latex','fontSize',11);
ylabel(s_y_label, 'Interpreter','Latex','fontSize',11);
grid on;
set(gca,'FontSize',11)
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize',11)
resize_fig('bias_plot');
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig');
export_fig(s_location_with_file_name, '-pdf','-transparent');

%% ACC
h = figure;
resize_fig( 'bias_plot' );
s_title = 'Accelerometer Bias Estimates';
s_y_label = '$\boldmath{\hat{b}}^b_a[\mathrm{m/s^2}$]';
s_fig_file_name = 'pva_acc_bias_est';

h1 = plot(time, bias_hat_acc(:,1), 'b' );
hold on
plot([time(1), time(end)], [bias_acc(1), bias_acc(1)], '--b' )
h2 = plot(time, bias_hat_acc(:,2), 'r' );
plot([time(1), time(end)], [bias_acc(2), bias_acc(2)], '--r' )
h3 = plot(time, bias_hat_acc(:,3), 'g' );
plot([time(1), time(end)], [bias_acc(3), bias_acc(3)], '--g' )
hold off
% title(s_title)
xlabel(s_xlabel, 'Interpreter','Latex','fontSize',11);
ylabel(s_y_label, 'Interpreter','Latex','fontSize',11);
legend([h1 h2 h3],s_legend, 'Interpreter','Latex','location','best','fontSize',11);
grid on;
set(gca,'FontSize',11)
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize',11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig');
export_fig(s_location_with_file_name, '-pdf','-transparent');

% xi
h = figure;
resize_fig( 'bias_plot' );
% s_y_label = {'$\xi_x$', '$\xi_y$', '$\xi_z$'};
s_y_label = {'$\mathbf{\xi}$ $\mathrm{[m/s^2]}$'};
s_title =  '$\xi$';
s_legend = {'$\xi_x$','$\xi_y$','$\xi_z$'};
% title(s_title, 'Interpreter','Latex')
plot(time, xi_hat(:,1),'b')
hold on;
plot(time, xi_hat(:,2),'--r')
plot(time, xi_hat(:,3),':g','Linewidth',0.3)
plot([time(1), time(end)], [0.05, 0.05], '--k')
plot([time(1), time(end)], [-0.05, -0.05], '--k')
hold off;
xlabel(s_xlabel, 'Interpreter','Latex','fontSize',11);
ylabel(s_y_label, 'Interpreter','Latex','fontSize',11);
legend( s_legend, 'Interpreter','Latex','fontSize',11)
grid on;
set(gca,'FontSize',11)

% subplot(3,1,1)
% plot(time, xi_hat(:,1))
% xlabel(s_xlabel);
% ylabel(s_y_label(1), 'Interpreter','Latex');
% % title(s_title, 'Interpreter','Latex')
% grid on;
% set(gca,'FontSize',11)
% 
% subplot(3,1,2)
% plot(time, xi_hat(:,2))
% xlabel(s_xlabel);
% ylabel(s_y_label(2), 'Interpreter','Latex');
% grid on;
% set(gca,'FontSize',11)
% 
% subplot(3,1,3)
% plot(time, xi_hat(:,3))
% xlabel(s_xlabel);
% ylabel(s_y_label(3), 'Interpreter','Latex');
% grid on;
% set(gca,'FontSize',11)

%[ax4,h3] = suplabel(s_title  ,'t');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize',11)
s_fig_file_name = 'pva_xi_estimates';
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig');
export_fig(s_location_with_file_name, '-pdf','-transparent');

%% Sigma
h = figure;
resize_fig( 'bias_plot' );
s_y_label = '$\mathbf{\hat \sigma}$ $\mathrm{[rad/s]}$';
s_legend = {'$\hat \sigma_x$','$\hat \sigma_y$','$\hat \sigma_z$'};
plot(time, sigma(:,1))
hold on
plot(time, sigma(:,2),'--r')
plot(time, sigma(:,3),':g','Linewidth',0.3)
xlabel(s_xlabel, 'Interpreter','Latex','fontSize',11);
legend(s_legend, 'Interpreter','Latex','fontSize',11)
ylabel(s_y_label, 'Interpreter','Latex','fontSize',11);
grid on;
set(gca,'FontSize',11)
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize',11)
s_fig_file_name = 'sigma';
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');

s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig');
export_fig(s_location_with_file_name, '-pdf','-transparent');

%% GNSS bias

index = find(gnss_z_bias_status_out, 1);
gnss_bias_detected_time_1 = time(index);
gnss_bias_detected_time_2 = time(index);
gnss_bias_detected_time_3 = time(index);

y_1 = [0 bias_gnss(index,1)];
y_2 = [0 bias_gnss(index,2)];
y_3 = [0 bias_gnss(index,3)];
if max(y_1) >= max(y_2) && max(y_1) >= max(y_1)
    y = y_1;
    gnss_bias_detected_time = gnss_bias_detected_time_1;
    k = 1;
elseif max(y_2) >= max(y_1) && max(y_2) >= max(y_3)
    y = y_2;
    gnss_bias_detected_time = gnss_bias_detected_time_2;
    k = 2;
elseif max(y_3) >= max(y_1) && max(y_3) >= max(y_2)
    y = y_3;
    gnss_bias_detected_time = gnss_bias_detected_time_3;
    k = 3;
end
t = [gnss_bias_detected_time gnss_bias_detected_time];

s_y_label = {'$\hat{b}_{\textrm{GNSS},z}^n$ [m]'};

    h = figure;
    resize_fig( 'bias_plot_2' );
    plot(time, y_1_out(:,3));
    hold on
    plot(time, bias_gnss,'r')
    if size(y, 2) > 1
        s_legend = {'$p_{\textrm{GNSS},z}$','$\hat b_{\textrm{GNSS 1},z}$', '$\hat b_{\textrm{GNSS 2},z}$', '$\hat b_{\textrm{GNSS 3},z}$','Bias Detected', 'Bias Detection Threshold'};
        plot(t, y,'*g')
    else
        s_legend = {'$p_{\textrm{GNSS},z}$','$\hat b_{\textrm{GNSS 1},z}$', '$\hat b_{\textrm{GNSS 2},z}$', '$\hat b_{\textrm{GNSS 3},z}$', 'Bias Detection Threshold'};
    end
    plot(time, gnss_bias_threshold(:,1), '--g')
    plot(time, -gnss_bias_threshold, '--g')
%     plot(t, y,'r.','markersize',11)
%     plot(t, y ,'c')
    xlabel(s_xlabel);
    ylabel(s_y_label, 'Interpreter','Latex');
    legend(s_legend, 'Location', 'best', 'Interpreter','Latex');
    grid on;
    set(gca,'FontSize',11)
if size(y, 2) > 1
    figureHandle = gcf;
    set(findall(figureHandle,'type','text'),'fontSize',11)
    s_fig_file_name = 'gnss_bias_estimate';
    s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
%     export_fig(s_fig_path_and_name, '-pdf','-transparent');
    s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
    saveas(h,s_location_with_file_name,'fig');
    export_fig(s_location_with_file_name, '-pdf','-transparent');
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    h = figure;
    resize_fig( 'bias_plot_2' );
    plot(time, y_1_out(:,3));
    hold on
    plot(time, bias_gnss','r')
    plot(t, y,'*g ')
    plot(time, gnss_bias_threshold, '--k')
    plot(time, -gnss_bias_threshold, '--k')
    plot(t, y,'--g')
    hold off;
    xlabel(s_xlabel, 'Interpreter','Latex','fontSize',11);
    ylabel(s_y_label, 'Interpreter','Latex','fontSize',11);
%     legend(s_legend, 'Location', 'NorthOutside', 'Interpreter','Latex');
    legend(s_legend, 'Location', 'best', 'Interpreter','Latex','fontSize',11);
    grid on;
    set(gca,'FontSize',11)    

    detection_time = time(index) - 700;
    x_min = time(index) - detection_time;
    x_max = time(index) + 5;
    y_min = bias_gnss(index, k) - 1.3*gnss_bias_threshold(index)*sign(max(bias_gnss(:, k)));
    y_max = bias_gnss(index, k) + 1.3*gnss_bias_threshold(index)*sign(max(bias_gnss(:, k))); 
    if y_max > y_min && x_max > x_min
        axis([x_min x_max y_min y_max]);
    end
    figureHandle = gcf;
    set(findall(figureHandle,'type','text'),'fontSize',11)
    s_fig_file_name = 'gnss_bias_estimate_zoomed';
    s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
%     export_fig(s_fig_path_and_name, '-pdf','-transparent');
    s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
    saveas(h,s_location_with_file_name,'fig');
    export_fig(s_location_with_file_name, '-pdf','-transparent');
end

%     figure
%     resize_fig('bias_plot');
%     s_y_label = '$\boldmath{\hat{b}}_{\mathrm{GNSS},z}$ [m]';
%     s_fig_file_name = 'GNSS_3DOF_est';
% 
%     subplot(3,1,1)
%     plot(time, y_1_out(:,1) - x_out(:,1), 'b')  
%     hold on;
%     plot(time, gnss_6DOF_bias_status_out(:,1), 'r') 
%     hold off;
%     grid on;
%     legend({'$p_{GNSS,x}$','$\hat b_x$'},'Location','EastOutside', 'Interpreter','Latex');
%     set(gca,'FontSize',11)
%     figureHandle = gcf;
%     set(findall(figureHandle,'type','text'),'fontSize',11)
%     
%     subplot(3,1,2)
%     plot(time, y_1_out(:,2) - x_out(:,2), 'b')
%     hold on;
%     plot(time, gnss_6DOF_bias_status_out(:,2), 'r') 
%     hold off;
%     grid on;
%     legend({'$p_{GNSS,y}$','$\hat b_y$'},'Location','EastOutside', 'Interpreter','Latex')
%     ylabel(s_y_label, 'Interpreter','Latex');
%     set(gca,'FontSize',11)
%     figureHandle = gcf;
%     set(findall(figureHandle,'type','text'),'fontSize',11)
%     
%     subplot(3,1,3)
%     plot(time, y_1_out(:,3) - x_out(:,3), 'b') 
%     hold on;
%     plot(time, gnss_6DOF_bias_status_out(:,3), 'r')
%     hold off;
%     grid on;
%     xlabel(s_xlabel, 'Interpreter','Latex');
%     
%     legend({'$p_{GNSS,z}$','$\hat b_z$'},'Location','EastOutside', 'Interpreter','Latex')
%     grid on;
%     set(gca,'FontSize',11)
%     figureHandle = gcf;
%     set(findall(figureHandle,'type','text'),'fontSize',11)
%     
%     s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
%     export_fig(s_fig_path_and_name, '-pdf','-transparent');
set(0,'defaulttextinterpreter','Latex')
end