function [tau_out, eta_int_out] = controller(time, dt, eta_r, q_hat, omega_hat, p_hat, v_hat, omega_b, vessel_model_params)  
    persistent tau M D eta_int eta_d nu_d eta_r_lf f_control k_controller
    
    if isempty( M ) || isempty( D )
        M = zeros(3,3); D = zeros(3,3);
        M(1:2,1:2) = vessel_model_params.M(1:2, 1:2);
        M(2,3) = vessel_model_params.M(2,6);
        M(3,2) = M(2,3);
        M(3,3) = vessel_model_params.M(6,6);

        D(1:2,1:2) = vessel_model_params.D(1:2, 1:2);
        D(2,3) = vessel_model_params.M(2,6);
        D(3,2) = D(2,3);
        D(3,3) = vessel_model_params.D(6,6);
    end
    if isempty( eta_int ) || isempty( eta_d) || isempty( nu_d ) || isempty( eta_r_lf )
       eta_int = [0 0 0]'; 
       eta_d = p_hat;
       nu_d = [0 0 0]';
       eta_r_lf = [0 0 0]';
    end
    if isempty( f_control), f_control = 10; end 
    if isempty( k_controller ), k_controller = f_control; end
    f_s = 1/dt;
    h = dt;
    
    [phi_hat, theta_hat, psi_hat] = q2euler(q_hat);
    R_bn = Rzyx(0, 0, psi_hat);
    eta_hat = [p_hat(1:2)' psi_hat]';
    nu_hat = [v_hat(1:2)' omega_hat(3)]';

    % reference model with velocity saturation
    omega_n_d = 0.1; 
    omega_n_d_psi = 0.1;
    
    zeta_d = 0.9;
    zeta_d_psi = 0.9;   
    Omega_d = diag( [omega_n_d omega_n_d omega_n_d_psi] );
    Delta_d = diag( [zeta_d zeta_d zeta_d_psi]);
    
    max_nu_d_dot_r = [0.5 0.5 rad2deg(0.5)];
    nu_max = [1 1 deg2rad(1)]';

    % controller
    zeta = 1;
    omega_n = omega_b/sqrt( 1 - 2*zeta^2 + sqrt( 4*zeta^4 - 4*zeta^2 + 2 ) );
    K_m = 0*M;
    K_p = (M + K_m)*omega_n^2;
    K_d = 2*zeta*omega_n*(M + K_m) - D;
    K_i = omega_n/10*K_p;
    % DP nonlinear PID contoller with reference model
    if k_controller == round(f_s/ f_control)
        % lowpass filter
        eta_r_lf_dot = Omega_d*(eta_r - eta_r_lf);
        eta_r_lf = eta_r_lf + h*f_control*eta_r_lf_dot;
        
        nu_d_dot_r = Omega_d^2*eta_r_lf;
%         nu_d_dot_r = sat_nu_dot_r( nu_d_dot_r, max_nu_d_dot_r);
       
        nu_d_dot = nu_d_dot_r - Omega_d^2*eta_d - 2*Delta_d*Omega_d*nu_d; 
        nu_d = nu_d + h*f_control*nu_d_dot;
        nu_d = sat_nu_d( nu_d, nu_max );
        
        eta_d_dot = nu_d;
        eta_d = eta_d + h*f_control*eta_d_dot;
        
        eta_tilde = eta_hat - eta_d;
        nu_tilde = nu_hat - nu_d;
        eta_int = eta_int + h*f_control*eta_tilde;
%         tau = - R_bn'*K_p*eta_tilde - R_bn'*K_d*nu_hat - R_bn'*K_i*eta_int;
        tau = - R_bn'*(K_p*eta_tilde + K_d*nu_tilde + R_bn'*K_i*eta_int);
        
        k_controller = 0;
    end
    k_controller = k_controller + 1;
    tau_out = [tau(1:2)' 0 0 0 tau(3)]';
    eta_int_out = K_i*eta_int;
end