function [status, gnss_bias_z, gnss_bias_3DOF, eta_n_out, nu_n_out] = gnss_bias( dt, p_hat, v_hat, gnss, ....
    time, q_hat, vessel_model_params, tau, tau_wave, omega_hat, ....
    gnss_bias_threshold, GNSS_available )

    h = dt;
    
    persistent bias_z bias_detected 
    if isempty(bias_z)
        bias_z = 0;
        bias_detected = 0;
    end

    T_bias = 1;
    k_b = 1;
%     bias_z_dot = -1/T_bias*bias_z + k_b*(0 - p_hat(3));
    bias_z_dot = -1/T_bias*bias_z + k_b*(gnss(3));
    % euler
    bias_z = bias_z + h*bias_z_dot;
    gnss_bias_z = bias_z;
      
    persistent bias T_inv M D eta nu bias_nu %D_inv p % M_inv
    if isempty(M) || isempty(D) || isempty(bias_nu) 
       bias = [0 0]';
       bias_nu = [ 0 0 0]';
       M = zeros(3,3); D = zeros(3,3);
       M(1:2,1:2) = vessel_model_params.M(1:2, 1:2);
       M(2,3) = vessel_model_params.M(2,6);
       M(3,2) = M(2,3);
       M(3,3) = vessel_model_params.M(6,6);
       
       D(1:2,1:2) = vessel_model_params.D(1:2, 1:2);
       D(3,3) = vessel_model_params.D(6,6);
    end
    
    persistent reset_counter model_update_counter
    if isempty( reset_counter ) 
        reset_counter = 0;
        model_update_counter = 0;
    end
    
    [phi, theta, psi] = quat2euler(q_hat);
    R_bn = Rzyx(0, 0, psi); 
    if isempty(eta) || isempty(nu)
        eta = [p_hat(1:2)' psi]';
        nu_b = R_bn'*v_hat;
        nu = [nu_b(1:2)', omega_hat(3)]';
    end
    if model_update_counter == 10
        nu(3) = omega_hat(3);
        u = [tau(1:2)' tau(6)]' + [tau_wave(1:2) tau_wave(6)]';   
        % Combined forward and backwards Euler integration
        nu_dot = M\(u - D*nu);
        nu = nu + h*nu_dot;  
        eta_dot = R_bn*nu + bias_nu ;
        eta = eta + h*eta_dot;
    
        model_update_counter = 0;
    end
    model_update_counter = model_update_counter + 1;
   
    if GNSS_available == 1
        reset_counter = reset_counter + 1;
        T = diag([100 100 10000]);
        K = diag([10 10 0]);
        y_tilde = p_hat - eta; 
        % dette stemmer ikke matematisk, ...
        bias_nu_dot = - T\bias_nu + K*y_tilde;
        bias_nu = bias_nu + h*bias_nu_dot;
    end
    if abs(bias_z) > gnss_bias_threshold ||  bias_detected == 1
        status = 4;
        bias_detected = 1;
    elseif abs(bias_z) < gnss_bias_threshold && bias_detected == 0
        status = 0;
    end
    
%     if reset_counter == 50 && status == 0 
%         %eta(1:2) = eta(1:2) + h*k_GNSS*(gnss(1:2) - eta(1:2));
%         eta(1:2) = p_hat(1:2);
%         v_hat_b = R_bn'*v_hat;
%         nu(1:2) = v_hat_b(1:2);
%         reset_counter = 0;
%     end
    eta_n_out = eta; 
    nu_n_out = R_bn*nu;
    
    diff = p_hat(1:2)- eta(1:2);
    T_bias = 0.5;
    if isempty(T_inv)
        T_inv = diag([1/T_bias 1/T_bias]);
    end
    k_b = 10;
    bias_dot = -T_inv*bias + k_b*diff;
    bias = bias + h*bias_dot;
    gnss_bias_3DOF = [bias(1:2)' bias_z]';
end
% gnss_z_bias = bias_prev;
% z_dot = 0;
% z_hat_dot = p_hat(3);
% diff = z_hat_dot - z_dot;
% K = 1.1;
% gnss_z_bias_dot = K*diff;
% 
% gnss_z_bias = gnss_z_bias + dt*gnss_z_bias_dot;


%         z_dot = 0;
%         z = 0;
    % working for detection
%     z_hat_dot = gnss(3);
%     diff_dot = z_hat_dot;
%     % wave
%     K = 2;
%     % notwave
% %     K = 5;
%     K_d = h*K;
%     gnss_z_bias_dot = K_d*diff_dot;

%     T_bias = 0.01;
%     k_b = 10;
%     bias_z_dot = -1/T_bias*bias_z + k_b*(gnss(3) - bias_z);
%     % euler
%     bias_z = bias_z + h*bias_z_dot;
%     gnss_bias_z = bias_z;

    % full bias detection with fast adaptation
%     persistent bias T_inv
%     if isempty(bias) 
%         bias = [0 0 0]';
%     end
%     T_bias = 1;
%     if isempty(T_inv)
%         T_inv = diag([1/T_bias 1/T_bias 1/T_bias]);
%     end
%     bias_dot = -T_inv*bias + k_b*(( gnss - p_hat) - bias);
%     bias = bias + h*bias_dot;
%     gnss_bias_3DOF = [bias(1:2)' bias_z]';

%     R_bn = Rquat(q_hat);
%     v = D_inv*(-M*f_IMU(1:2) - G*p + tau(1:2) );%+ tau_wave(1:2) );
%     v_ext = [v' 0]';
%     p_dot = R_bn*v_ext;
%     p = p + h*p_dot(1:2); 
%     
%     % output
%     p_vessel_mod = p;
%   