function [gnss_pos_voted, GNSS_status, weighted_var] = gnss_check_vote_and_averaging( ....
    GNSS_1, var_pos_ref_1, status_GNSS_1,....
    GNSS_2, var_pos_ref_2, status_GNSS_2,....
    GNSS_3, var_pos_ref_3, status_GNSS_3)
    
    % enabling and desabling sensors
    global time
    persistent status_1_prev status_2_prev status_3_prev T_f b_sensor_disabled y_f_w y_w_prev time_smooting_0 time_smooting
    if isempty(status_1_prev), status_1_prev = status_GNSS_1; end
    if isempty(status_2_prev), status_2_prev = status_GNSS_2; end
    if isempty(status_3_prev), status_3_prev = status_GNSS_3; end
    if isempty(T_f), T_f = 1; end
    if isempty(y_w_prev), y_w_prev = [0 0 0]'; end
    if isempty(y_f_w), y_f_w = [0 0 0]'; end
    if isempty(time_smooting_0), time_smooting_0 = 0; end
    if isempty(time_smooting), time_smooting = 0; end
    
    % case 1: normal, case 2: wild point, case 3: freeze, case 4: high
    % variance
    
    switch status_GNSS_1
        case 1
            status_1 = 1;
        case {2, 3, 4, 5, -1}
            status_1 = 0;
    end 
    switch status_GNSS_2
        case 1
            status_2 = 1;
        case {2, 3, 4, 5,-1}
            status_2 = 0;
    end
    switch status_GNSS_3
        case 1
            status_3 = 1;
        case {2, 3, 4, 5, -1}
            status_3 = 0;
    end
    % voting threshold
    threshold = 2.5;
    
    if status_1 + status_2 + status_3 >= 1
        [status_vote, status_1, status_2, status_3] = GNSS_voting( GNSS_1, GNSS_2, GNSS_3, threshold, status_1, status_2, status_3);
        
        % weighting
        [y_w, P_w] = weighted_ls( GNSS_1, var_pos_ref_1 , status_1, GNSS_2, var_pos_ref_2 , status_2, GNSS_3, var_pos_ref_3 , status_3);
        
        % check is sensor just got disabled
        if ( status_1_prev == 1 && (status_1_prev ~= status_1) ) || ( status_3_prev == 1 && (status_2_prev ~= status_2) ) || ( status_3_prev == 1 && (status_3_prev ~= status_3) )
            b_sensor_disabled  = true;
            time_smooting_0 = time;
        else
           b_sensor_disabled  = false; 
        end
        
        % is just disabled or smooting is occuring
        if b_sensor_disabled || (time_smooting > 0 && time_smooting < 5)
           b_smooth_change = true; 
        else
           b_smooth_change = false; 
        end
        
        % smooting. possible step in voted pos
        if b_smooth_change == true
            alpha = 0.5;
            y_f_w = y_w*(1 - exp(-alpha*time_smooting)) + y_w_prev*exp(-alpha*time_smooting); 
            gnss_pos_voted = y_f_w;
            % gjr dynamisk for samplings freq
            time_smooting = time - time_smooting_0 + 0.2;  
        else
            gnss_pos_voted = y_w;
            y_w_prev = y_w;
            time_smooting = 0; 
        end
        
        GNSS_status = status_vote;
        weighted_var = [P_w(1,1) P_w(2,2) P_w(3,3)]';
    elseif status_1 + status_2 + status_3 == 0
        gnss_pos_voted = [0 0 0]';
        GNSS_status.status = -1;
        GNSS_status.msg = '123';
        weighted_var = [0 0 0]';
    else
        disp(status_1), disp(status_2), disp(status_3)
        error('Voting error. Unknown source');
    end
    
    status_1_prev = status_1; status_2_prev = status_2; status_3_prev = status_3;
end