function [a_meas_b, omega_meas_b, m_meas_b, biases] = imu( k, N, x, x_dot_r, nu_c_n, m_n, imu_info)
% [a_meas_b, omega_meas_b, psi_m] = IMU(acc_b, omega_b)
%
% Author:   Torleiv H. Bryne
% Date:     28th January 2013
% ________________________________________________________________
%
% The function thats accelarations and angular velocity from a vessel
% simulator model. The function returns the three axis measurments of the
% modeled acceleromteres and gyroscopes. The function also returns the
% measured heading dicrectly instead of the magnetic field.
% 
% E-mail: torleiv.haland.bryne@gmail.com
   
    persistent b_a_b b_gyro_b b_mag_b w_acc w_gyro w_mag
    if isempty(b_a_b ) || isempty(b_gyro_b) || isempty(b_mag_b)
        b_a_b = imu_info.b_a_b;
        b_gyro_b = imu_info.b_gyro_b;
        b_mag_b = imu_info.b_mag_b;
        
        w_acc_std =  imu_info.w_acc_b;
        w_gyro_std = imu_info.w_gyro_b;
        w_mag_std  = imu_info.w_mag_b;
        
        w_acc = normrnd(0, w_acc_std, N+1, 3);
        w_gyro = normrnd(0, w_gyro_std, N+1, 3);
        w_mag  = normrnd(0, w_mag_std, N+1, 3);
    end
    biases = struct('b_a_b', b_a_b, 'b_gyro_b', b_gyro_b, 'b_mag_b', b_mag_b); 

    % gravity in NED
    g_n = [0 0 9.81]';
    
    % attitude
    phi = x(4); theta = x(5); psi = x(6);
    R_bn = Rzyx(phi, theta, psi);
    
    % angular velocity ( irroation current)
    omega_b = x(10:12); 
    
    % acceleration calulation in BODY
    v_c_n = nu_c_n(1:3);
    v_c_b = R_bn'*v_c_n;
    v_r_b_dot = x_dot_r(1:3);
    S = Smtrx(omega_b);
    v_c_b_dot = - S*v_c_b;
    
    v_b_dot = v_r_b_dot + v_c_b_dot;
    
    w_acc_b =  w_acc(k,:)';
    w_gyro_b = w_gyro(k,:)';
    w_mag_b  = w_mag(k,:)';
    
    a_meas_b = v_b_dot - R_bn'*g_n + b_a_b + w_acc_b;
    omega_meas_b = omega_b + b_gyro_b + w_gyro_b; 
    m_meas_b = R_bn'*m_n + b_mag_b + w_mag_b;
end