%% Master thesis simulation init
%%
clc
close all
s = dbstatus;
save('myBreakpoints.mat', 's');
clear all
load('myBreakpoints.mat');
dbstop(s);
echo off
global time

% disp('Press eny key to conintue')
%% Configuration
[sim_info, GNSS_info, imu_info] = read_config_files;
f_s = sim_info.sampling_freq;
f_GNSS = GNSS_info.GNSS_sampling_freq;
dt = 1/f_s;
b_plot_bias_corr = false;

%% Initial states:  
x = sim_info.states;
eta_r = [x(1) x(2) x(6)]';

% Initial GNSS measurments
y_1 = x(1:3); y_2 = x(1:3); y_3 = x(1:3);

% Initial IMU measurments
a_1 = [0 0 0]'; a_2 = [0 0 0]'; a_3 = [0 0 0]';
omega_1 = [0 0 0]'; omega_2 = [0 0 0]'; omega_3 = [0 0 0]';
m_1 = [0 0 0]';
m_n = sim_info.m_field_n;
%% Vessel spesific data
vessel_model_params = get_vessel_params( sim_info.vessel_type );

%% Initial control input 
commanded_tau = sim_info.tau;
tau = commanded_tau;

%%  Memmory Allocation
N = round(sim_info.t_final / dt); % number of samples
time_out = zeros( N+1, 1);

commanded_tau_out = zeros( N+1, length( commanded_tau ) );
tau_out = commanded_tau_out;
x_out = zeros( N+1, length( x ));  
zero_mat = zeros(6, 6);
y_1_out = zeros( N+1, length( x(1:3) ));  y_2_out = y_1_out; y_3_out = y_1_out;
a_b_meas_1 = zeros( N+1, 3 ); a_b_meas_2 = a_b_meas_1; a_b_meas_3 = a_b_meas_1;
omega_b_meas_1 = zeros( N+1, 3 ); 
omega_b_meas_2 = zeros( N+1, 3 ); 
omega_b_meas_3 = zeros( N+1, 3 );
m_b_meas_1 = zeros( N+1, 3 ); m_b_meas_2 = m_b_meas_1; m_b_meas_3 = m_b_meas_1;
p_hat_out = zeros( N+1, 3 ); v_hat_out = zeros( N+1, 3 ); 
q_hat_out = [ones(N+1, 1) zeros( N+1, 3 )]; 
bias_g_hat_out = zeros( N+1, 3 ); bias_a_hat_out = zeros( N+1, 3 );
b_g_out = ones( N+1, 3 ); b_a_out = ones( N+1, 3 );
xi_hat_out = zeros( N+1, 3 );
sigma_out = zeros( N+1, 3 );

gnss_z_bias_out = zeros( N+1, 3);

gnss_z_bias_status_out = zeros( N+1, 1);
gnss_bias_threshold_out = zeros( N+1, 1);
xypsi_model_out = zeros( N+1, 3);
uvr_model_out = zeros( N+1, 3);
bias_model_out = zeros( N+1, 3);
nu_c_out = zeros( N+1, 6);
omega_hat_out = zeros( N+1, 3); 

xi_wave_hat_out = zeros( N+1, 3);
eta_int_out = zeros( N+1, 3);
acc_diff_filter_out = zeros( N+1, 2); acc_diff_out = zeros( N+1, 2);
g_t_1_out = zeros( N+1, 2); g_t_2_out = zeros( N+1, 2);
status_GNSS_1_out = zeros( N+1, 1); status_GNSS_2_out = zeros( N+1, 1); status_GNSS_3_out = zeros( N+1, 1); GNSS_status_out = zeros( N+1, 1); GNSS_status_msg = cell(N+1,1);
var_pos_ref_1_out = zeros( N+1, 3); var_pos_ref_2_out = zeros( N+1, 3); var_pos_ref_3_out = zeros( N+1, 3); gnss_var_w_out = zeros(N+1, 3);
gnss_pos_w_out = zeros(N+1, 3);

acc_model_out = zeros( N+1, 3);

%% Generate wave forces
calc_wave_flag = 0;
[tau_wave_out, wave_model_params] = wave_generator( dt, N, sim_info.power_w_wave, ...
    sim_info.vessel_type, vessel_model_params.M, calc_wave_flag );

%% Observer Initialization
p_hat = zeros(3,1); v_hat = zeros(3,1); q_hat = [1 0 0 0]';
bias_g_hat = zeros(3,1); bias_a_hat = zeros(3,1); xi_hat = zeros(3,1);
gnss_z_bias = 0;
gnss_bias_3_DOF = [ 0 0 0]';
status_gnss_bias = 0;

%global gnss_bias_threshold
gnss_bias_threshold = 0.8;
k_GNSS = 20;
bias_gnss_bias_3_DOF = [0 0 0]';
sigma = [0 0 0]';
status = 1;
bias_detected = 0;
xypsi_model = [0 0 0]'; uvr_model = xypsi_model; bias_model = xypsi_model;
omega_hat = [0 0 0]'; xi_wave_hat = [0 0 0]';
time_without_GNSS = 0;
b_use_current = 1; nu_c_n = zeros(6,1);
gnss_z_bias_1 = 0; gnss_z_bias_2 = 0; gnss_z_bias_3 = 0;
k_controller = 10; f_controller = 10;
eta_int = [0 0 0]';
status_GNSS_1 = 1;
status_GNSS_2 = 1;
status_GNSS_3 = 1;
GNSS_status.status = 1; GNSS_status.msg = 123;
gnss_pos_w = [0 0 0]'; gnss_var_w = [0 0 0]';
acc_model = [0 0 -9.81]';
var_pos_ref_1  = [0 0 0]'; var_pos_ref_2  = [0 0 0]'; var_pos_ref_3  = [0 0 0]';

b_run_quest = 1;
b_estimate_acc_bias = 0;
b_plot_NPO = false;
fault_time_tau = 3000;


%% Main Loop 
tic
for k = 1:N+1,
    time = (k-1)*dt; 
    % Storing to memory
    time_out(k,:) = time; 
    tau_out(k,:) = tau';
    commanded_tau_out(k,:) = commanded_tau'; 
    x_out(k,:) = x;
    nu_c_out(k, :) = nu_c_n'; 
    y_1_out(k,:) = y_1; y_2_out(k,:) = y_2; y_3_out(k,:) = y_3;
    a_b_meas_1(k, :) = a_1; a_b_meas_2(k, :) = a_2; a_b_meas_3(k, :) = a_3; m_b_meas_1(k, :) = m_1;
    omega_b_meas_1(k, :) = omega_1; omega_b_meas_2(k, :) = omega_2; omega_b_meas_3(k, :) = omega_3;
    p_hat_out(k, :) = p_hat; v_hat_out(k, :) = v_hat; q_hat_out(k, :) = q_hat; 
    bias_g_hat_out(k, :) = bias_g_hat; bias_a_hat_out(k, :) = bias_a_hat; xi_hat_out(k, :) = xi_hat;
    sigma_out(k, :) = sigma; omega_hat_out(k,:) = omega_hat';
    gnss_z_bias_out(k,:) = [gnss_z_bias_1 gnss_z_bias_2 gnss_z_bias_3];
    gnss_z_bias_status_out(k) = status_gnss_bias;
    gnss_bias_threshold_out(k) = gnss_bias_threshold;
    xypsi_model_out(k, :) = xypsi_model'; uvr_model_out(k, :) = uvr_model'; bias_model_out(k, :) = bias_model';
    xi_wave_hat_out(k, :) = xi_wave_hat'; eta_int_out(k, :) = eta_int';
    status_GNSS_1_out(k) = status_GNSS_1; status_GNSS_2_out(k) = status_GNSS_2; status_GNSS_3_out(k) = status_GNSS_3; 
    GNSS_status_out(k) = GNSS_status.status; 
    GNSS_status_msg{k,1} = GNSS_status.msg;
    var_pos_ref_1_out(k, :) = var_pos_ref_1'; var_pos_ref_2_out(k, :) = var_pos_ref_2'; var_pos_ref_3_out(k, :) = var_pos_ref_3';
    gnss_var_w_out(k, :) = gnss_var_w'; gnss_pos_w_out(k, :) = gnss_pos_w';
    acc_model_out(k,:)  = acc_model';  
    
    % fault generation
    [fault_signal_1, fault_signal_2, fault_signal_3, fault_signal_acc] = fault_generator;
    
    % control input
    omega_b = 0.1;
    if time >= 1500;
        eta_r = [100 100 0]';
    end
    if time >= 2500
        eta_r = [100 100 deg2rad(40)]';
    end
    
    if time >= 25
        [commanded_tau, eta_int] = controller(time, dt, eta_r, q_hat, omega_hat, p_hat, v_hat, omega_b, vessel_model_params);
    end
    if time >= fault_time_tau
       time_tau = time - fault_time_tau;  
       tau = commanded_tau*0.60 + commanded_tau*0.4*exp(-0.5*time_tau); 
    else
       tau = commanded_tau;
    end
    [x, x_dot_r, nu_c_n] = generate_vessel_data( dt, k, N, x, ....
        vessel_model_params, tau, tau_wave_out(k,:)', b_use_current); 
    
    % obtaining IMU measurments,    
    [a_1, omega_1, m_1, biases_1] = imu( k, N, x, x_dot_r, nu_c_n, m_n, imu_info); %[a_2, omega_2, m_2, biases_2] = imu(acc_b, omega_b, Omega_bn, m_n, imu_info);%[a_3, omega_3, m_3, biases_3] = imu(acc_b, omega_b, Omega_bn, m_n, imu_info);
    % if model bases acc shoud be utelized instead
    if fault_signal_acc == 1
       a_1 = acc_model; 
    end
    
    % obtaining GNSS measurments
    y_v = [0 0 0]'; % gnss velocity
    if k_GNSS == round( f_s/f_GNSS )
        xyz_vessel = x(1:3);
        [GNSS_meas_obtained_1, y_1, var_GNSS_1] = GNSS( xyz_vessel, k, N, GNSS_info, 1, fault_signal_1 );
        [GNSS_meas_obtained_2, y_2, var_GNSS_2] = GNSS( xyz_vessel, k, N, GNSS_info, 2, fault_signal_2 );
        [GNSS_meas_obtained_3, y_3, var_GNSS_3] = GNSS( xyz_vessel, k, N, GNSS_info, 3, fault_signal_3 );
        k_GNSS = 0;
        
%         % Indivudal GNSS signal check
        [status_GNSS_1, var_pos_ref_1, pos_ref_number, gnss_z_bias_1] = pos_ref_sys_check( y_1, 1, dt, GNSS_meas_obtained_1, gnss_bias_threshold );
        [status_GNSS_2, var_pos_ref_2, pos_ref_number, gnss_z_bias_2] = pos_ref_sys_check( y_2, 2, dt, GNSS_meas_obtained_2, gnss_bias_threshold );
        [status_GNSS_3, var_pos_ref_3, pos_ref_number, gnss_z_bias_3] = pos_ref_sys_check( y_3, 3, dt, GNSS_meas_obtained_3, gnss_bias_threshold );
       
        if status_GNSS_1 == 4 || status_GNSS_2 == 4 || status_GNSS_3 == 4
            status_gnss_bias = 1;
        end
     
         % Total GNSS check, voting and averaging
        [gnss_pos_w, GNSS_status, gnss_var_w] = gnss_check_vote_and_averaging( ....
            y_1, var_pos_ref_1, status_GNSS_1,.... 
            y_2, var_pos_ref_2, status_GNSS_2,....  
            y_3, var_pos_ref_3, status_GNSS_3);
        b_GNSS_sampled = true;
        if GNSS_status.status ~= -1;   
            time_without_GNSS = 0;
        end
    else 
        b_GNSS_sampled = false;
    end
    k_GNSS = k_GNSS + 1;
    time_without_GNSS = time_without_GNSS + dt;
    
    % Grip et al. observer with modifications
    [p_hat, v_hat, q_hat, xi_hat, bias_g_hat, bias_a_hat, sigma, dot_b_hat, omega_hat] = ....
    observer_thb( dt, GNSS_status.status, a_1, omega_1, m_1,.... 
    gnss_pos_w, gnss_var_w, y_v, b_GNSS_sampled, f_GNSS, m_n, gnss_z_bias_1,.... 
    xypsi_model(1:2), uvr_model(1:2), var_GNSS_1 );
    
    % Nonlinear passive observer 
    b_perform_innovation = true;
    [xypsi_model, uvr_model, bias_model, xi_wave_hat, acc_model] = nonlinear_passive_observer( .... 
        dt, b_GNSS_sampled, gnss_pos_w, p_hat, v_hat, q_hat, vessel_model_params, wave_model_params, tau, omega_hat, b_perform_innovation, time);
end
t_stop = toc;
s_output = strcat('Time to compute PVA estimates: ', sprintf('%g', t_stop));
disp(s_output);

set(0,'defaulttextinterpreter','Latex')
save_file = 'time_data'; save(save_file, 'time_out');
save_file = 'states_data'; save(save_file, 'x_out');
save_file = 'tau_data'; save(save_file, 'tau_out', 'eta_int_out');
save_file = 'current_data';  save(save_file, 'nu_c_out');
save_file = 'gyro_bias_data'; save(save_file, 'bias_g_hat_out');
save_file = 'acc_bias_data'; save(save_file, 'bias_a_hat_out');
save_file = 'biases_actual_data'; save(save_file, 'biases_1')
save_file = 'GNSS_data'; save(save_file, 'y_1_out','y_2_out','y_3_out', 'gnss_pos_w_out' , 'status_GNSS_1_out', 'status_GNSS_2_out', 'status_GNSS_3_out', 'var_pos_ref_1_out', 'var_pos_ref_2_out', 'var_pos_ref_3_out', 'gnss_var_w_out', 'GNSS_status_out');
save_file = 'IMU_data'; save(save_file, 'a_b_meas_1' , 'omega_b_meas_1', 'm_b_meas_1');
save_file = 'pva_hat_data'; save(save_file, 'p_hat_out', 'v_hat_out', 'q_hat_out','xi_hat_out','sigma_out');
save_file = 'GNSS_bias_data'; save(save_file, 'gnss_z_bias_out', 'gnss_z_bias_status_out', 'gnss_bias_threshold_out');
save_file = 'npo_data'; save(save_file, 'xypsi_model_out', 'uvr_model_out','bias_model_out', 'xi_wave_hat_out');

% Plotting
global s_Folder;
s_Folder = copy_data_to_new_folder;
vessel_plot(time_out, x_out, tau_out, tau_wave_out, nu_c_out, eta_int_out);
GNSS_plot(time_out, x_out(:,1:3), y_1_out, y_2_out, y_3_out, status_GNSS_1_out, status_GNSS_2_out, status_GNSS_3_out, var_pos_ref_1_out, var_pos_ref_2_out, var_pos_ref_3_out, gnss_var_w_out, GNSS_status_out, gnss_pos_w_out);
pva_est_plot(time_out, p_hat_out, v_hat_out, q_hat_out, x_out, xypsi_model_out, gnss_pos_w_out, nu_c_out, b_plot_NPO);
vessel_model_plot(time_out, xypsi_model_out, uvr_model_out, bias_model_out, xi_wave_hat_out, p_hat_out, v_hat_out, q_hat_out, y_1_out, nu_c_out );
imu_plot(time_out, a_b_meas_1 , omega_b_meas_1, m_b_meas_1, bias_a_hat_out, bias_g_hat_out, b_plot_bias_corr);
bias_plot(time_out, bias_g_hat_out, bias_a_hat_out, biases_1, xi_hat_out, sigma_out, gnss_z_bias_out, gnss_z_bias_status_out, gnss_bias_threshold_out, y_1_out, x_out);
thurst_and_acc_plot(time_out, a_b_meas_1, acc_model_out)