function [eta_n_out, nu_b_out, bias_n_out, xi_hat_out, acc_model_out] = nonlinear_passive_observer( ....
    dt, b_GNSS_sampled, gnss, p_hat, v_hat, q_hat, vessel_model_params, wave_model_params, tau_6_dof, omega_hat, b_perform_innovation, time)

    h = dt*10;
    f_GNSS = 5;
    k_GNSS = 1/h / f_GNSS;
    
    [phi_hat, theta_hat, psi_hat] = quat2euler(q_hat);
    R_bn = Rzyx(0, 0, psi_hat); 
    
    persistent initialized bias_n_pred xi_pred eta_n_pred nu_b_pred ....
        T M D C_w I mat_0 A_w E_w K_1 K_2 K_3 K_4 k_NPO nu_dot nu_dot_RB
    %% Initialization
    if isempty(initialized) || initialized == 0 
        % conter
        k_NPO = 10;
        
        % Vessel Parameters
        M = zeros(3,3); D = zeros(3,3); M(1:2,1:2) = vessel_model_params.M(1:2, 1:2);
        M(2,3) = vessel_model_params.M(2,6); M(3,2) = M(2,3); M(3,3) = vessel_model_params.M(6,6);

        D(1:2,1:2) = vessel_model_params.D(1:2, 1:2); D(3,3) = vessel_model_params.D(6,6);
        %% Wave Parameters 
        omega = wave_model_params.omega; lambda = wave_model_params.lambda;
        Omega = diag([omega, omega, omega]); Lambda = diag([lambda, lambda, lambda]);
%         K_w = diag([k_w, k_w, k_w]);
        
        mat_0 = zeros(3); I = eye(3);
        A_w = [ mat_0       I; 
                -Omega^2    -2*Lambda*Omega];
%         E_w = [mat_0; K_w];
        C_w = [mat_0 I];
        xi_pred = [0 0 0 0 0 0]';

        bias_n_pred = [0 0 0]'; 
        T = diag([1000 1000 1000]);
        eta_n_pred = [p_hat(1:2)' psi_hat]';
        nu_b_pred = R_bn'*v_hat;
        nu_b_pred = [nu_b_pred(1:2)', omega_hat(3)]';
        initialized = 1;
        
        zeta_ni = 1;
        lambda_i = lambda;
        omega_0_i = omega;
%         omega_c_i = 1.1*omega_0_i;
        omega_c_i = 1.2255*omega_0_i;        %1.2255
        k_1_1 = -2*(zeta_ni - lambda_i)*omega_c_i/omega_0_i;
        k_1_2 = -2*(zeta_ni - lambda_i)*omega_c_i/omega_0_i; 
        k_1_3 = -2*(zeta_ni - lambda_i)*omega_c_i/omega_0_i;
        
        k_1_4 = 2*omega_0_i*(zeta_ni - lambda_i); 
        k_1_5 = 2*omega_0_i*(zeta_ni - lambda_i); 
        k_1_6 = 2*omega_0_i*(zeta_ni - lambda_i); 
        
        K_1 = [ diag([k_1_1 k_1_2 k_1_3]);
                diag([k_1_4 k_1_5 k_1_6]) ];
        
        k_2_1 = omega_c_i; k_2_2 = k_2_1; k_2_3 = k_2_1;
        K_2 = diag([k_2_1 k_2_2 k_2_3]);
        
%         k_4_1 = 500; k_4_2 = 500; k_4_3 = 1000;
%         k_4_1 = 10000; k_4_2 = 10000; k_4_3 = 10000;
        k_4_1 = 0.1; k_4_2 = 0.1; k_4_3 = 0.01;
%         k_4_1 = 10; k_4_2 = 10; k_4_3 = 10;
        K_4 = diag([k_4_1 k_4_2 k_4_3]);
        
        K_3 = 0.5*K_4;%diag([k_3_1 k_3_2 k_3_3]);
        
        nu_dot_RB = [0 0 0]';
        nu_dot = [ 0 0 0 ]';
    end
    
    if b_GNSS_sampled == true && b_perform_innovation == true
        k = [1 1 1]';
    else 
        k = [0 0 1]';
    end
    if k_NPO == 10
        y = [gnss(1:2)' psi_hat]';
        y_hat = eta_n_pred + C_w*xi_pred;
        y_tilde = k.*(y - y_hat);

        % update wave model
        K_d_1 = h*k_GNSS*K_1;
        xi_hat = xi_pred + K_d_1*y_tilde;           % corrector
        xi_dot = A_w*xi_hat; 
        xi_pred = xi_hat + h*xi_dot;                % predictor
        
        % update bias
        K_3_e = K_3*(eye(3) + eye(3)*exp(-0.1*time)); 
        K_d_3 = h*k_GNSS*K_3_e;
        bias_n_hat = bias_n_pred + K_d_3*y_tilde;   % corrector
        bias_dot = [0 0 0]';
%         bias_dot = - T\bias_n_hat;
        bias_n_pred = bias_n_hat + h*bias_dot;      % predictor
        
        tau = [tau_6_dof(1:2)' tau_6_dof(6)]'; 
        
        K_d_4 = h*k_GNSS*R_bn'*K_4;                 
        nu_b_hat = nu_b_pred + K_d_4*y_tilde;       % corrector     
        nu_dot = M\(-D*nu_b_hat + tau);
%         nu_dot = M\(-D*nu_b_hat + R_bn'*bias_n_pred + tau);
        nu_b_pred = nu_b_hat + h*nu_dot;            % predictor

        K_2_d = h*k_GNSS*K_2;
        eta_n_hat = eta_n_pred + K_2_d*y_tilde;     % corrector
        eta_dot = R_bn*nu_b_pred + bias_n_pred;
%         eta_dot = R_bn*nu_b_pred;
%         eta_dot = R_bn*nu_b_hat + bias_n_hat;
        eta_n_pred = eta_n_hat + h*eta_dot;         % predictor
        
%         eta_n_out = eta_n_hat;
%         nu_b_out = nu_b_hat;
%         bias_n_out = bias_n_hat;
%         xi_hat_out = C_w*xi_hat;
       
        nu_dot_RB = M\(-D*(nu_b_pred + R_bn'*bias_n_pred)  + tau);
        k_NPO = 0;
    end
    k_NPO = k_NPO + 1;
    eta_n_out = eta_n_pred;
    nu_b_out = nu_b_pred;
    bias_n_out = bias_n_pred;
    xi_hat_out = C_w*xi_pred;
    
    nu_dor_r(1:2) = nu_dot(1:2);
    nu_dor_r(3) = - 9.81;
    acc_model_out = nu_dor_r' -Smtrx(omega_hat)*R_bn'*bias_n_pred;
end
%         xi_dot = A_w*xi_hat + K_1*y_tilde;
%         xi_hat = xi_hat + h*k_GNSS*xi_dot;
% 
%     %     bias_dot = - T\bias_n_hat + K_3*y_tilde;
%         bias_dot = K_3*y_tilde;
%         bias_n_hat = bias_n_hat + h*k_GNSS*bias_dot;
% 
%         tau = [tau_6_dof(1:2)' tau_6_dof(6)]'; 
% 
%         nu_dot = M\(-D*nu_b_hat + tau + R_bn'*K_4*y_tilde);
%     %     nu_dot = M\(-D*nu_b_hat + R_bn'*bias_n_hat + tau + R_bn'*K_4*y_tilde);
%         nu_b_hat = nu_b_hat + k_GNSS*h*nu_dot;
% 
%         eta_dot = R_bn*nu_b_hat + bias_n_hat + K_2*y_tilde;
%     %     eta_dot = R_bn*nu_b_hat + K_2*y_tilde;
%         eta_n_hat = eta_n_hat + h*eta_dot;
% 
%         eta_n_out = eta_n_hat;
%         nu_b_out = nu_b_hat;
%         bias_n_out = bias_n_hat;
%         xi_hat_out = C_w*xi_hat;

%     persistent reset_counter model_update_counter
%     if isempty( reset_counter ) 
%         reset_counter = 0;
%         model_update_counter = 0;
%     end
%     
%     
%     
%     if model_update_counter == 10
%         nu(3) = omega_hat(3);
%         u = [tau(1:2)' tau(6)]' + [tau_wave(1:2) tau_wave(6)]';   
%         % Combined forward and backwards Euler integration
%         nu_dot = M\(u - D*nu);
%         nu = nu + h*nu_dot;  
%         eta_dot = R_bn*nu + bias_nu ;
%         eta = eta + h*eta_dot;
%   
%         model_update_counter = 0;
%     end
%     model_update_counter = model_update_counter + 1;

%% 6 DOF ACC funka ikke
   
%     persistent acc_bias
%     if isempty(acc_bias)
%         acc_bias = [0 0 0]';
%     end
%     v = R_bn'*v_hat;
%     nu = [v' omega_hat']';
%     eta = [p_hat' phi_hat theta_hat theta_hat]';
%     J =  eulerang(phi_hat, theta_hat, psi_hat); 
%     nu_dot_raw = M_6DOF\(tau_6_dof - D_6DOF*nu - J\G_6DOF*eta);
%     acc_diff = a_acc(1:3) - nu_dot_raw(1:3);
%     
%     T_inv = 1/T_thurst*eye(3); 
%     
%     diff_dot = -T_inv*acc_bias + acc_diff;
%     acc_bias = acc_bias + h*diff_dot;
