function [status, var_pos_ref, pos_ref_number, gnss_bias_z] = pos_ref_sys_check( ....
    pos_meas, pos_ref_number, dt, GNSS_meas_obtained, gnss_bias_threshold)
    
    global time
    % status 1: normal
    % status 2: wild point
    % status 3: freeze
    % status 4: high variance
    % status 5: drift z axis
    % status -1: no measurement available
    
%     b_enable_bias_estimation = false;
    b_enable_bias_estimation = true;
    
    persistent buffer_1 buffer_2 buffer_3 bias_z_1 bias_z_2 bias_z_3 ....
        bias_detected_1 bias_detected_2 bias_detected_3....
        prev_status prev_status_1 prev_status_2 prev_status_3 ....
        c_fresh_signal c_fresh_signal_1 c_fresh_signal_2 c_fresh_signal_3
    size_meas_buffer = 25;
    pos_ref_freeze_limit = [0.35 0.35 0.4]'; % [m^2]
    pos_ref_hight_var_limit = [10 10 12]'; % [m^2]
    pos_ref_wild_point_limit = 7; % [m]
    h = dt*20;
    if isempty(buffer_1), buffer_1 = zeros(3, size_meas_buffer); end
    if isempty(buffer_2), buffer_2 = zeros(3, size_meas_buffer); end
    if isempty(buffer_3), buffer_3 = zeros(3, size_meas_buffer); end
    if isempty(bias_z_1), bias_z_1 = 0; bias_detected_1 = 0; end
    if isempty(bias_z_2), bias_z_2 = 0; bias_detected_2 = 0; end
    if isempty(bias_z_3), bias_z_3 = 0; bias_detected_3 = 0; end
    if isempty(prev_status_1) || isempty(prev_status_2) || isempty(prev_status_3) 
        prev_status = 1; prev_status_1 = 1; prev_status_2 = 1; prev_status_3 = 1; 
        c_fresh_signal = 0;
    end
    
    % creating retrieving buffer for current sensor
    switch pos_ref_number
        case 1 
            meas_buffer = buffer_1(:,2:end);
            bias_z = bias_z_1;
            bias_detected = bias_detected_1;
            prev_status = prev_status_1;
            c_fresh_signal = c_fresh_signal_1;
        case 2
            meas_buffer = buffer_2(:,2:end);
            bias_z = bias_z_2;
            bias_detected = bias_detected_2;
            prev_status = prev_status_2;
            c_fresh_signal = c_fresh_signal_2;
        case 3
            meas_buffer = buffer_3(:,2:end);
            bias_z = bias_z_3;
            bias_detected = bias_detected_3;
            prev_status = prev_status_3;
            c_fresh_signal = c_fresh_signal_3;
        otherwise
            error('No correct case');
    end
    % if position measurment is obtained 
    if GNSS_meas_obtained == true
        % signal processing starts when statistic is fully build up
        run_signal_check = (length(find(meas_buffer(1,:))) == length(meas_buffer)); 

        % if freeze has occured
        if prev_status == 2
            c_fresh_signal = 5;
        end
        if c_fresh_signal > 0
            c_fresh_signal = c_fresh_signal - 1;
            b_fresh_signals_in_buffer = false;
        else
            b_fresh_signals_in_buffer = true;
        end
        % outlier dection
        if run_signal_check == true && b_fresh_signals_in_buffer;
            [wild_point, status] = wild_point_check( meas_buffer, pos_meas, pos_ref_wild_point_limit );
            var_pos_ref = [-1 -1 -1]';
        else
            wild_point = false;
        end

        % insert current measurment in buffer
        if wild_point == false
            meas_buffer(:, end + 1) = pos_meas;
        elseif wild_point == 1
            meas_buffer(:, end + 1) = meas_buffer(:, end);
        end
        
        % variance calculation, freeze and high variance calulation
        if wild_point == false
            [var_pos_ref, status] = variance_calculation_and_check( ....
                meas_buffer, run_signal_check, pos_ref_freeze_limit, pos_ref_hight_var_limit); 
        end
%         % bias and drift detection
        if b_enable_bias_estimation == true && wild_point == false
            T_bias = 1*4;
            k_bias = 1/T_bias;
            bias_z_dot = -1/T_bias*bias_z + k_bias*(pos_meas(3));

            % euler
            bias_z = bias_z + h*bias_z_dot;
        end
        if abs(bias_z) > gnss_bias_threshold %||  bias_detected == 1
            status = 5;
            %bias_detected = 1;
        end
        gnss_bias_z = bias_z;

    elseif GNSS_meas_obtained == false;
        meas_buffer(:, end + 1) = [0 0 0]';
        status = -1;
        var_pos_ref = [-1, -1, -1]';
%         var_pos_ref = [1e6, 1e6, 1e6]';
    end
    
    % storing meas_buffer for spesific sensor
    switch pos_ref_number
        case 1 
            buffer_1 = meas_buffer;
            bias_z_1 = bias_z;
            bias_detected_1 = bias_detected;
            prev_status_1 = status;
            c_fresh_signal_1 = c_fresh_signal;
        case 2
            buffer_2 = meas_buffer;
            bias_z_2 = bias_z;
            bias_detected_2 = bias_detected;
            prev_status_2 = status;
            c_fresh_signal_2 = c_fresh_signal;
        case 3
            buffer_3 = meas_buffer;
            bias_z_3 = bias_z;
            bias_detected_3 = bias_detected;
            prev_status_3 = status;
            c_fresh_signal_3 = c_fresh_signal;
    end
end