function pva_est_plot(time, p_hat, v_hat, q_hat, states, xypsi_model_out, GNSS, nu_c_out, b_plot_NPO)

global s_Folder;

n = length(time); 
p = states(:,1:3);
v = states(:,7:9);
v_n = zeros(n, 3);
roll = states(:,4);
pitch = states(:,5); 
yaw = states(:,6);

phi_hat = zeros(n, 1);
theta_hat = zeros(n, 1);
psi_hat = zeros(n, 1);
q = zeros(n, 4);

for k = 1:length(v)
    R_bn = Rzyx( roll(k), pitch(k), yaw(k) );
    v_n(k,:) = (R_bn*v(k,:)')';
    q(k,:) = euler2q(roll(k), pitch(k), yaw(k) );
    [phi_hat(k), theta_hat(k),psi_hat(k)] = quat2euler(q_hat(k, :));
end
phi_hat_deg = rad2deg(phi_hat);
theta_hat_deg =  rad2deg(theta_hat);
psi_hat_deg = rad2deg(psi_hat);

phi_deg = rad2deg(roll);
theta_deg = rad2deg(pitch);
psi_deg = rad2deg(yaw);

%% North, East
% Meas, Est, Vessel
s_fig_file_name = 'north_east_plot_vessel';
s_label_x = 'East [m]';
s_label_y = 'North [m]';

h = figure;
resize_fig( 'bias_plot' );
plot(GNSS( 1:end, 2), GNSS( 1:end, 1),'b')
hold on;
plot(p_hat( 1:end, 2), p_hat( 1:end, 1), 'r')
if b_plot_NPO == true
%     plot(xypsi_model_out( 700*100:end, 2), xypsi_model_out( 700*100:end, 1), 'g')
    plot(xypsi_model_out( 1:end, 2), xypsi_model_out( 1:end, 1), 'g')
end
% plot(p( 700*100:end, 2), p( 700*100:end, 1),'b')
plot(p( 1:end, 2), p( 1:end, 1),'g')
hold off;
xlabel(s_label_x, 'Interpreter','Latex'); 
ylabel(s_label_y, 'Interpreter','Latex');
if b_plot_NPO == true
    s_legend = {....
    'Horizontal Position Estimate (GNSS/INS)',....
    'Horizontal Position Estimate (NPO)',....
    'Horizontal Vessel Position'};
else
    s_legend = {....
    'Horizontal Position Measurement GNSS',....
    'Horizontal Position Estimate (GNSS/INS)',....
    'Horizontal Vessel Position'};
end
legend(s_legend, 'location', 'NorthOutside', 'Interpreter','Latex');
grid on;
%axis equal
set(gca,'FontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
axis equal
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

% Meas, Est
s_fig_file_name = 'north_east_plot_meas';
h = figure;
resize_fig( 'simulator_plot' );
plot(GNSS(:, 2), GNSS( :, 1),'b') % plot(GNSS( 700*100:end, 2), GNSS( 700*100:end, 1),'b')
hold on;
plot(p_hat(:, 2), p_hat( :, 1), 'r') % plot(p_hat( 700*100:end, 2), p_hat( 700*100:end, 1), 'r')

if b_plot_NPO == true
    plot(xypsi_model_out( 700*100:end, 2), xypsi_model_out( 700*100:end, 1), 'g')
end
xlabel(s_label_x, 'Interpreter','Latex'); 
ylabel(s_label_y, 'Interpreter','Latex');
if b_plot_NPO == true
    s_legend = {....
    'Horizontal Position Measurement',....
    'Horizontal Position Estimate (GNSS/INS)',....
    'Horizontal Position Estimate (NPO)'};

else    
    s_legend = {....
    'Horizontal Position Measurement',....
    'Horizontal Position Estimate (GNSS/INS)'};
end
legend(s_legend, 'location', 'NorthOutside', 'Interpreter','Latex');
grid on;
set(gca,'FontSize',11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
axis equal
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')


%% Error e
s_fig_file_name = 'pva_estimation_error_pos';
h = figure;
resize_fig( 'simulator_plot' );

s_label_y = '$\mathbf{e} = \mathbf{p}^n_\mathrm{GNSS} - \mathbf{\hat{p}}^n$ [m]';
s_label_x = 'Time [s]';
s_legend = {'$e_x$','$e_y$','$e_z$'};

plot(time, GNSS(:,1) - p_hat(:,1), 'b')
hold on;
plot(time, GNSS(:,2) - p_hat(:,2), '--r')
plot(time, GNSS(:,3) - p_hat(:,3), ':g','linewidth', 0.3)
hold off
grid on;
legend(s_legend, 'Location', 'best', 'Interpreter','Latex');
xlabel(s_label_x, 'Interpreter','Latex'), 
ylabel(s_label_y, 'Interpreter','Latex');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

%% Position
s_x_label = 'Time [s]';
s_label_y_x = '$p_x^n$[m]'; s_label_y_y = '$p_y^n$[m]'; s_label_y_z = '$p_z^n$[m]';
% s_legend_x = {'$x_\mathrm{GNSS}$', '$\hat{x}$'};%, '$x$'};
% s_legend_y = {'$y_\mathrm{GNSS}$','$\hat{y}$'};%, '$y$'};
% s_legend_z = {'$z_\mathrm{GNSS}$', '$\hat{z}$'};%, '$z$'};
s_legend_x = {'$\hat{p}_x$', '$p_x$'};
s_legend_y = {'$\hat{p}_y$', '$p_y$'};
s_legend_z = {'$\hat{p}_z$', '$p_z$'};
h = figure;
resize_fig( 'pva_plot' );
s_fig_file_name = 'pva_est_pos';
subplot(3,1,1)
    %plot(time, GNSS(:,1), 'b')
    plot(time, p_hat(:,1), 'r')
    hold on
    plot(time, p(:,1), '--b')
%     plot(time, p(:,1), '--g')%,'LineWidth',2) 
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_x, 'Interpreter','Latex');
    legend(s_legend_x, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    hold off;
subplot(3,1,2)
%     plot(time, GNSS(:,2), 'b')
    plot(time, p_hat(:,2), 'r')
    hold on
    plot(time, p(:,2), '--b')
%     plot(time, p(:,2), '--g')%,'LineWidth',2) 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_y, 'Interpreter','Latex')
    legend(s_legend_y, 'Location', 'best', 'Interpreter','Latex')
    grid on
    set(gca,'FontSize', 11)
    hold off;
subplot(3,1,3)
%     plot(time, GNSS(:,3), 'b')
    plot(time, p_hat(:,3), 'r')
    hold on
    plot(time, p(:,3), '--b')
%     plot(time, p(:,3), '--g')%,'LineWidth',2) 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_z, 'Interpreter','Latex');
    legend(s_legend_z, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    hold off
%[ax4,h3] = suplabel('Position Estimates'  ,'t');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

%% Position error
s_x_label = 'Time [s]';
s_label_y = '$\mathbf{\tilde{p}}_x^n$[m]';
s_legend = {'$\tilde{p}_x$','$\tilde{p}_y$','$\tilde{p}_z$'};
s_fig_file_name = 'pva_est_pos_tilde';
h = figure;
resize_fig( 'bias_plot' );
plot(time, p(:,1) - p_hat(:,1))
hold on;
plot(time, p(:,2) - p_hat(:,2), '--r')
plot(time, p(:,3) - p_hat(:,3), ':g')
hold off;
xlabel(s_x_label, 'Interpreter','Latex'), 
ylabel(s_label_y, 'Interpreter','Latex');
legend(s_legend, 'Location', 'best', 'Interpreter','Latex');
grid on
set(gca,'FontSize', 11)
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

%% Velocity 
s_fig_file_name = 'pva_est_velo';
s_label_y_x = '$v_x^n$[m/s]'; s_label_y_y = '$v_y^n$[m/s]'; s_label_y_z = '$v_z^n$[m/s]';
s_legend_x = {'$\hat{v}_x$', '$v_x$'};
s_legend_y = {'$\hat{v}_y$', '$v_y$'};
s_legend_z = {'$\hat{v}_z$', '$v_x$'};
h = figure;
resize_fig( 'pva_plot' );
subplot(3,1,1)
    plot(time, v_hat(:,1), 'r')
    hold on
    plot(time, v_n(:,1) + nu_c_out(:,1), '--b')%,'LineWidth',2) %, '--b') 
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_x , 'Interpreter','Latex')
    legend(s_legend_x, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    
subplot(3,1,2)
    plot(time, v_hat(:,2), 'r')
    hold on
    plot(time, v_n(:,2) + nu_c_out(:,2), '--b')%,'LineWidth',2) %, '--b')
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_y, 'Interpreter','Latex')
    legend(s_legend_y, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    hold off
subplot(3,1,3)
    plot(time, v_hat(:,3), 'r')
    hold on
    plot(time, v_n(:,3) + nu_c_out(:,3), '--b')%,'LineWidth',2) %, '--b')
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_z, 'Interpreter','Latex')
    legend(s_legend_z, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    hold off
%[ax4,h3] = suplabel('Velocity Estimates'  ,'t');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

%% Velocity Error
s_fig_file_name = 'pva_est_velo_tilde';
s_label_y = '$\mathbf{\tilde{v}}$[m/s]';
s_legend = {'$\tilde{v}_x$','$\tilde{v}_y$','$\tilde{v}_z$'};
h = figure;
resize_fig( 'bias_plot' );
plot(time, v_n(:,1) + nu_c_out(:,1) - v_hat(:,1))
hold on;
plot(time, v_n(:,2) + nu_c_out(:,2) - v_hat(:,2), '--r')
plot(time, v_n(:,3) + nu_c_out(:,3) - v_hat(:,3), ':g')
hold off;
xlabel(s_x_label, 'Interpreter','Latex'), 
ylabel(s_label_y, 'Interpreter','Latex')
legend(s_legend, 'Location', 'best', 'Interpreter','Latex');
grid on
set(gca,'FontSize', 11)
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')


%% Attiude
% Quaternions
s_label_y_eta = '$\eta$'; 
s_label_y_eps_1 = '$\epsilon_1$'; 
s_label_y_eps_2 = '$\epsilon_2$'; 
s_label_y_eps_3 = '$\epsilon_3$'; 
s_legend_eta = {'$\hat{\eta}$', '$\eta$'};
s_legend_eps_1 = {'$\hat{\epsilon}_1$', '$\epsilon_1$'};
s_legend_eps_2 = {'$\hat{\epsilon}_2$', '$\epsilon_2$'};
s_legend_eps_3 = {'$\hat{\epsilon}_3$', '$\epsilon_3$'};
h = figure;
resize_fig( 'pva_plot' );
subplot(4,1,1)
    hold on
    plot(time, q_hat(:,1), 'r')
    plot(time, q(:,1), '--b') 
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_eta, 'Interpreter','Latex');
    legend(s_legend_eta, 'Location', 'NorthEast', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
    
subplot(4,1,2)
    hold on
    plot(time, q_hat(:,2), 'r')
    plot(time, q(:,2), '--b') 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_eps_1, 'Interpreter','Latex');
    legend(s_legend_eps_1, 'Location', 'NorthEast', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)

subplot(4,1,3)
    hold on
    plot(time, q_hat(:,3), 'r')
    plot(time, q(:,3), '--b') 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_eps_2, 'Interpreter','Latex');
    legend(s_legend_eps_2, 'Location', 'NorthEast', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)

subplot(4,1,4)
    hold on
    plot(time, q_hat(:,4), 'r')
    plot(time, q(:,4), '--b') 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_eps_3, 'Interpreter','Latex');
    legend(s_legend_eps_3, 'Location', 'NorthEast', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
%[ax4,h3] = suplabel('Attiude Estimates, Quaternions'  ,'t');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_file_name = 'pva_est_attiude_quat';
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

% Euler angles
s_label_y_x = '$\phi$[deg]'; s_label_y_y = '$\theta$[deg]'; s_label_y_z = '$\psi$[deg]';
s_legend_x = {'$\hat{\phi}$', '$\phi$'};
s_legend_y = {'$\hat{\theta}$', '$\theta$'};
s_legend_z = {'$\hat{\psi}$', '$\psi$'};
h = figure;
resize_fig( 'pva_plot' );
subplot(3,1,1)
    hold on
    plot(time, phi_hat_deg, 'r')
    plot(time, phi_deg, '--b') 
    hold off
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_x, 'Interpreter','Latex')
    legend(s_legend_x, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)  
subplot(3,1,2)
    hold on
    plot(time, theta_hat_deg, 'r')
    plot(time, theta_deg, '--b') 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_y, 'Interpreter','Latex')
    legend(s_legend_y, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)

subplot(3,1,3)
    hold on
    plot(time, psi_hat_deg, 'r')
    plot(time, psi_deg, '--b') 
    xlabel(s_x_label, 'Interpreter','Latex'), 
    ylabel(s_label_y_z, 'Interpreter','Latex')
    legend(s_legend_z, 'Location', 'best', 'Interpreter','Latex');
    grid on
    set(gca,'FontSize', 11)
%[ax4,h3] = suplabel('Attiude Estiamte, Euler Angles'  ,'t');
figureHandle = gcf;
set(findall(figureHandle,'type','text'),'fontSize', 11)
s_fig_file_name = 'pva_est_attiude_euler';
s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
% export_fig(s_fig_path_and_name, '-pdf','-transparent');
s_location_with_file_name = strcat( s_Folder,'\', s_fig_file_name);
saveas(h,s_location_with_file_name,'fig')
export_fig(s_location_with_file_name, '-pdf','-transparent')

