function [sim_info, GNSS_info, IMU_info] = read_config_files

%% Reading Simulation and Simulator Data
file_id = fopen('config_files/rig_simulation_config.txt');
param = textscan(file_id,'%s%f32', 'headerLines', 1);
fclose(file_id);

var_name = param{1};
var_value = param{2};

if size(var_name,1) == size(var_value,1) && size(var_name,2) == size(var_value,2) == 0
   error('Check config file for errors! Number of variable names are not consistent with variables') 
end

vessel_index = var_value(1);
switch vessel_index
    case 0
        vessel_type = 'Supply vessel';
    case 1
        vessel_type = 'Rig';
end

for k = 1:length(var_value)
    if ~isnan(var_value(k)) || strcmp(var_name(k),'Vessel_type')
        eval([var_name{k} '= ' num2str(var_value(k)) ';'])
    end
end

eta = [x; y; z; deg2rad(roll); deg2rad(pitch); deg2rad(yaw)];
nu = [u; v; w; p; q; r];
tau = [X Y Z K M N]';
power_w_wave = [...
    wave_noise_power_surge;
    wave_noise_power_sway;
    wave_noise_power_heave;
    wave_noise_power_roll; 
    wave_noise_power_pitch;
    wave_noise_power_yaw];

m_n = [m_n_x m_n_y m_n_z]';

sim_info = struct('vessel_type', vessel_type,....
    'sampling_freq', sampling_freq, 't_final', t_final',....
    'states', [eta' nu']', 'tau', tau, ....
    'power_w_wave', power_w_wave, 'm_field_n', m_n);

%% Reading GNSS information
 
file_id = fopen('config_files/GNSS_config.txt');
param = textscan(file_id,'%s%f32', 'headerLines', 1);
fclose(file_id);

var_name = param{1};
var_value = param{2};

if size(var_name,1) == size(var_value,1) && size(var_name,2) == size(var_value,2) == 0
   error('Check config file for errors! Number of variable names are not consistent with variables') 
end

for k = 1:length(var_value)
    if ~isnan(var_value(k))
        eval([var_name{k} '= ' num2str(var_value(k)) ';']);
    end
end

GNSS_bias = [GNSS_bias_x GNSS_bias_y GNSS_bias_z]'; 

GNSS_info = struct(....
    'GNSS_sampling_freq', GNSS_sampling_freq,....
    'GNSS_normal_std_weight', GNSS_normal_std_weight,....
    'GNSS_high_noise_std_weight', GNSS_high_noise_std_weight,....
    'GNSS_bias', GNSS_bias ....
    );

%% Reading IMU information
 
file_id = fopen('config_files/imu_config.txt');
param = textscan(file_id,'%s%f32', 'headerLines', 1);
fclose(file_id);

var_name = param{1};
var_value = param{2};

if size(var_name,1) == size(var_value,1) && size(var_name,2) == size(var_value,2) == 0
   error('Check config file for errors! Number of variable names are not consistent with variables') 
end

for k = 1:length(var_value)
    if ~isnan(var_value(k))
        eval([var_name{k} '= ' num2str(var_value(k)) ';']);
    end
end

w_acc_b = w_acc_b_std;
w_gyro_b = w_gyro_b_std;
w_mag_b = w_mag_b_std;
 
b_a_b =     [b_a_b_x    b_a_b_y     b_a_b_z]';
b_gyro_b =  [b_gyro_b_x b_gyro_b_y  b_gyro_b_z]';
b_mag_b =   [b_mag_b_x  b_mag_b_y   b_mag_b_z]';

IMU_info = struct(....
    'w_acc_b', w_acc_b ,....
    'w_gyro_b', deg2rad(w_gyro_b) ,....
    'w_mag_b', w_mag_b,....
    'b_a_b', b_a_b, ....
    'b_gyro_b', deg2rad(b_gyro_b),....
    'b_mag_b', b_mag_b ....				
    );